/*
 * Copyright (c) 2006-2011 Maskat Project.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Maskat Project - initial API and implementation
 */
package jp.sf.maskat.ui.editors.layout;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.AlignmentRetargetAction;
import org.eclipse.gef.ui.actions.DeleteRetargetAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.RedoRetargetAction;
import org.eclipse.gef.ui.actions.UndoRetargetAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.LabelRetargetAction;
import org.eclipse.ui.actions.RetargetAction;

/**
 * レイアウトエディタにエディタアクションをコントリビュートするクラスです
 *  
 * 以下のエディタアクションを追加します。
 * 
 * ・削除
 * ・元に戻す
 * ・やり直し
 * ・コピー
 * ・切り取り
 * ・貼り付け
 * ・並び替え（右、横中央、右、上、縦中央、下）
 *
 */
public class LayoutGraphicalEditorActionContributor extends ActionBarContributor {

	/**
     * {@inheritDoc}
     */
	protected void buildActions() {
		addRetargetAction(new DeleteRetargetAction());
		addRetargetAction(new UndoRetargetAction());
		addRetargetAction(new RedoRetargetAction());
		addRetargetAction(createAction(ActionFactory.COPY.getId(), "Copy",
				ISharedImages.IMG_TOOL_COPY,
				ISharedImages.IMG_TOOL_COPY_DISABLED));
		addRetargetAction(createAction(ActionFactory.CUT.getId(), "Cut",
				ISharedImages.IMG_TOOL_CUT, ISharedImages.IMG_TOOL_CUT_DISABLED));
		addRetargetAction(createAction(ActionFactory.PASTE.getId(), "Paste",
				ISharedImages.IMG_TOOL_PASTE,
				ISharedImages.IMG_TOOL_PASTE_DISABLED));

		addRetargetAction(new AlignmentRetargetAction(PositionConstants.LEFT));
		addRetargetAction(new AlignmentRetargetAction(PositionConstants.CENTER));
		addRetargetAction(new AlignmentRetargetAction(PositionConstants.RIGHT));
		addRetargetAction(new AlignmentRetargetAction(PositionConstants.TOP));
		addRetargetAction(new AlignmentRetargetAction(PositionConstants.MIDDLE));
		addRetargetAction(new AlignmentRetargetAction(PositionConstants.BOTTOM));
	}

	/**
	 * エディタアクションを作成します
	 * 
	 * @param id アクションID
	 * @param name アクション名
	 * @param imgId イメージID
	 * @param disabledImgId 無効時のイメージID
	 * @return 作成したエディタアクション
	 */
	private RetargetAction createAction(String id, String name, String imgId,
			String disabledImgId) {
		RetargetAction action = new LabelRetargetAction(id, name);
		ISharedImages sharedImages = PlatformUI.getWorkbench()
				.getSharedImages();
		action.setImageDescriptor(sharedImages.getImageDescriptor(imgId));
		action.setDisabledImageDescriptor(sharedImages
				.getImageDescriptor(disabledImgId));
		return action;
	}

    /**
     * {@inheritDoc}
     */
	protected void declareGlobalActionKeys() {
	}

    /**
     * {@inheritDoc}
     */
	public void contributeToToolBar(IToolBarManager toolBarManager) {
		toolBarManager.add(getAction(ActionFactory.DELETE.getId()));
		toolBarManager.add(getAction(ActionFactory.UNDO.getId()));
		toolBarManager.add(getAction(ActionFactory.REDO.getId()));
		toolBarManager.add(getAction(ActionFactory.COPY.getId()));
		toolBarManager.add(getAction(ActionFactory.CUT.getId()));
		toolBarManager.add(getAction(ActionFactory.PASTE.getId()));

		getPage().addPartListener(
				(RetargetAction) getAction(ActionFactory.COPY.getId()));
		getPage().addPartListener(
				(RetargetAction) getAction(ActionFactory.CUT.getId()));
		getPage().addPartListener(
				(RetargetAction) getAction(ActionFactory.PASTE.getId()));

		toolBarManager.add(new Separator());

		/*
		 * 水平方向の整列アクションの追加
		 */
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_LEFT));
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_CENTER));
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_RIGHT));
		toolBarManager.add(new Separator());
		
		/*
		 * 垂直方向の整列アクションの追加
		 */
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_TOP));
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_MIDDLE));
		toolBarManager.add(getActionRegistry().getAction(
				GEFActionConstants.ALIGN_BOTTOM));

	}
}
